#!/usr/bin/env python3
import multiprocessing
import re
import os
import subprocess
import sys

# 'printable name': 'maketarget'
MAKE_TARGETS = (
        'report-duplicates EXIT=y',
        'validation'
)
ITSADICT = re.compile(r"^[a-z]{3}-[a-z]{3}")
ERR_PATTERN = re.compile(r'^(E[0-9][0-9]?:\s+)')

def call(cmdline):
    if isinstance(cmdline, str):
        cmdline = cmdline.split(' ')
    return subprocess.Popen(cmdline, stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)

#pylint: disable=inconsistent-return-statements
def validate_dictionary(directory):
    """directory == dictname"""
    orig_cwd = os.getcwd()
    os.chdir(directory)
    try:
        procs = [] # process handles
        for target in MAKE_TARGETS:
            procs.append(call(['make',  '--silent',
                    '--no-print-directory'] + target.split(' ')))
        # agregate results
        results = []
        for handle in procs:
            text = handle.communicate()
            ret = handle.wait()
            if ret:
                text = '\n'.join(t.decode(sys.getdefaultencoding())
                    for t in text)
                lines = text.split('\n')[:-1] # strip last \n
                if lines[-1].startswith('make:'):
                    text = '\n'.join(lines[:-1]) # strip make failure message
                if text:
                    results.append(text)
        if results:
            return (directory, results)
    finally:
        os.chdir(orig_cwd)

def check_all_dicts(dictionaries):
    """ToDo: assumes to live in directory where list of supplied dicts
    exists."""
    with multiprocessing.Pool() as p:
        results = p.map(validate_dictionary, [d for d in dictionaries])
        return dict(filter(bool, results))


def main():
    res = check_all_dicts(d for d in os.listdir('.') if ITSADICT.search(d))
    for name, err in res.items():
        print(name + ':')
        for text in err:
            errcode = ERR_PATTERN.search(text)
            if errcode:
                errcode = errcode.groups()[0]
            if errcode:
                text = text[len(errcode[0])+1:]
                errcode = (errcode + ' ').ljust(4)
            else:
                errcode = ''
            text = '\n  '.join(text.split('\n')).lstrip()
            print("- {}{}".format(errcode, text))
    print("\nYou can learn more about errors starting with 'E<num>' by typing "
            "`make description E=<num>`.")

if __name__ == '__main__':
    main()

